@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2010 Igalia S.L.
+ * Copyright (C) 2010, 2011 Igalia S.L.
+ * Copyright (C) 2011 Intel Corporation.
  *
  * Contact: Iago Toral Quiroga <itoral@igalia.com>
  *
@@ -25,6 +26,7 @@
 #include <config.h>
 
 #include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include <string.h>
 #include <gconf/gconf-client.h>
 
@@ -69,14 +71,16 @@ GRL_LOG_DOMAIN_STATIC(test_ui_log_domain);
 
 #define WINDOW_TITLE "Grilo Test UI"
 
+#define NOTIFICATION_TIMEOUT 5
+
 #define BROWSER_MIN_WIDTH   320
 #define BROWSER_MIN_HEIGHT  400
 
 #define METADATA_MIN_WIDTH  320
 #define METADATA_MIN_HEIGHT 400
 
-#define BROWSE_CHUNK_SIZE   100
-#define BROWSE_MAX_COUNT    (2 * BROWSE_CHUNK_SIZE)
+#define BROWSE_CHUNK_SIZE   50
+#define BROWSE_MAX_COUNT    200
 
 enum {
   OBJECT_TYPE_SOURCE = 0,
@@ -127,9 +131,13 @@ typedef struct {
   GtkTreeModel *browser_model;
   GtkWidget *metadata;
   GtkTreeModel *metadata_model;
+  GtkWidget *statusbar;
+  guint statusbar_context_id;
 } UiView;
 
 typedef struct {
+  gboolean changes_notification;
+
   /* Keeps track of our browsing position and history  */
   GList *source_stack;
   GList *container_stack;
@@ -151,6 +159,12 @@ typedef struct {
 } UiState;
 
 typedef struct {
+  enum OperationType {
+    OP_TYPE_BROWSE,
+    OP_TYPE_SEARCH,
+    OP_TYPE_QUERY,
+    OP_TYPE_MULTI_SEARCH,
+  } type;
   guint offset;
   guint count;
   gchar *text;
@@ -172,6 +186,7 @@ static const gchar *ui_definition =
 "  <menu name='FileMenu' action='FileMenuAction' >"
 "   <menuitem name='Authorize Flickr' action='AuthorizeFlickrAction' />"
 "   <menuitem name='Shutdown plugins' action='ShutdownPluginsAction' />"
+"   <menuitem name='Changes notification' action='ChangesNotificationAction' />"
 "   <menuitem name='Quit' action='QuitAction' />"
 "  </menu>"
 " </menubar>"
@@ -186,6 +201,13 @@ static void authorize_flickr_cb (GtkAction *action);
 static void shutdown_plugins_cb (GtkAction *action);
 static void shutdown_plugins (void);
 
+static void changes_notification_cb (GtkToggleAction *action);
+static void content_changed_cb (GrlMediaSource *source,
+                                GrlMedia *media,
+                                GrlMediaSourceChangeType change_type,
+                                gboolean location_unknown,
+                                gpointer data);
+
 static GtkActionEntry entries[] = {
   { "FileMenuAction", NULL, "_File" },
   { "AuthorizeFlickrAction", GTK_STOCK_CONNECT, "_Authorize Flickr", NULL,
@@ -196,6 +218,11 @@ static GtkActionEntry entries[] = {
     "Quit", G_CALLBACK (quit_cb) }
 };
 
+static GtkToggleActionEntry toggle_entries[] = {
+  { "ChangesNotificationAction", GTK_STOCK_FIND, "_Changes notification", NULL,
+    "ChangesNotification", G_CALLBACK (changes_notification_cb), FALSE }
+};
+
 static void
 quit_cb (GtkAction *action)
 {
@@ -214,6 +241,38 @@ shutdown_plugins_cb (GtkAction *action)
   shutdown_plugins ();
 }
 
+static void
+changes_notification_cb (GtkToggleAction *action)
+{
+  GList *sources, *source;
+  GrlPluginRegistry *registry;
+
+  ui_state->changes_notification = gtk_toggle_action_get_active (action);
+
+  registry = grl_plugin_registry_get_default ();
+  sources = grl_plugin_registry_get_sources (registry, FALSE);
+  for (source = sources; source; source = g_list_next (source)) {
+    if (grl_metadata_source_supported_operations (GRL_METADATA_SOURCE (source->data)) &
+        GRL_OP_NOTIFY_CHANGE) {
+      if (ui_state->changes_notification) {
+        grl_media_source_notify_change_start (GRL_MEDIA_SOURCE (source->data),
+                                              NULL);
+        g_signal_connect (GRL_MEDIA_SOURCE (source->data),
+                          "content-changed",
+                          G_CALLBACK (content_changed_cb),
+                          NULL);
+      } else {
+        grl_media_source_notify_change_stop (GRL_MEDIA_SOURCE (source->data),
+                                             NULL);
+        g_signal_handlers_disconnect_by_func (source->data,
+                                              content_changed_cb,
+                                              NULL);
+      }
+    }
+  }
+  g_list_free (sources);
+}
+
 static GtkTreeModel *
 create_browser_model (void)
 {
@@ -467,7 +526,13 @@ metadata_cb (GrlMediaSource *source,
 			   view->metadata_model);
 
   if (error) {
-    g_critical ("Error: %s", error->message);
+    if (g_error_matches (error,
+                         GRL_CORE_ERROR,
+                         GRL_CORE_ERROR_OPERATION_CANCELLED)) {
+      GRL_DEBUG ("Operation cancelled");
+    } else {
+      g_critical ("Error: %s", error->message);
+    }
     return;
   }
 
@@ -476,15 +541,18 @@ metadata_cb (GrlMediaSource *source,
     keys = grl_data_get_keys (GRL_DATA (media));
     i = keys;
     while (i) {
-      const GValue *g_value = grl_data_get (GRL_DATA (media), i->data);
-      gchar *value = g_value ? g_strdup_value_contents (g_value) : "";
-      gtk_list_store_append (GTK_LIST_STORE (view->metadata_model), &iter);
-      gtk_list_store_set (GTK_LIST_STORE (view->metadata_model),
-			  &iter,
-			  METADATA_MODEL_NAME, GRL_METADATA_KEY_GET_NAME (i->data),
-			  METADATA_MODEL_VALUE, value,
-			  -1);
-      GRL_DEBUG ("  %s: %s", GRL_METADATA_KEY_GET_NAME (i->data), value);
+      if (grl_data_key_is_known (GRL_DATA (media), i->data)) {
+        const GValue *g_value = grl_data_get (GRL_DATA (media), i->data);
+        gchar *value = g_value ? g_strdup_value_contents (g_value) : "";
+        gtk_list_store_append (GTK_LIST_STORE (view->metadata_model), &iter);
+        gtk_list_store_set (GTK_LIST_STORE (view->metadata_model),
+                            &iter,
+                            METADATA_MODEL_NAME,
+                            GRL_METADATA_KEY_GET_NAME (i->data),
+                            METADATA_MODEL_VALUE, value,
+                            -1);
+        GRL_DEBUG ("  %s: %s", GRL_METADATA_KEY_GET_NAME (i->data), value);
+      }
       i = g_list_next (i);
     }
 
@@ -515,31 +583,46 @@ operation_started (GrlMediaSource *source, guint operation_id,
   ui_state->cur_op_source = source;
   ui_state->cur_op_id = operation_id;
   ui_state->multiple = multiple;
+
+  /* Set busy cursor */
+  GdkCursor *cursor;
+  cursor = gdk_cursor_new (GDK_WATCH);
+  gdk_window_set_cursor(gtk_widget_get_window (view->window), cursor);
+  gdk_cursor_destroy(cursor);
 }
 
 static void
 operation_finished (void)
 {
   ui_state->op_ongoing = FALSE;
+
+  /* Set default cursor */
+  gdk_window_set_cursor(gtk_widget_get_window (view->window), NULL);
 }
 
 static void
-browse_cb (GrlMediaSource *source,
-	   guint browse_id,
-	   GrlMedia *media,
-	   guint remaining,
-	   gpointer user_data,
-	   const GError *error)
+browse_search_query_cb (GrlMediaSource *source,
+			guint op_id,
+			GrlMedia *media,
+			guint remaining,
+			gpointer user_data,
+			const GError *error)
 {
   gint type;
   const gchar *name;
   GtkTreeIter iter;
   GdkPixbuf *icon;
   OperationState *state = (OperationState *) user_data;
-  guint next_browse_id;
+  guint next_op_id;
 
   if (error) {
-    g_critical ("Error: %s", error->message);
+    if (g_error_matches (error,
+                         GRL_CORE_ERROR,
+                         GRL_CORE_ERROR_OPERATION_CANCELLED)) {
+      GRL_DEBUG ("Operation cancelled");
+    } else {
+      g_critical ("Error: %s", error->message);
+    }
   }
 
   state->count++;
@@ -571,44 +654,77 @@ browse_cb (GrlMediaSource *source,
 			-1);
 
     g_object_unref (media);
-    gdk_pixbuf_unref (icon);
+    if (icon) {
+      gdk_pixbuf_unref (icon);
+    }
   }
 
   if (remaining == 0) {
     /* Done with this chunk, check if there is more to browse */
     if (ui_state->op_ongoing &&
-	ui_state->cur_op_id == browse_id &&
+	ui_state->cur_op_id == op_id &&
 	media != NULL) {
       /* Operation is still valid, so let's go */
       state->offset += state->count;
       if (state->count >= BROWSE_CHUNK_SIZE &&
 	  state->offset < BROWSE_MAX_COUNT) {
+	GRL_DEBUG ("operation (%d) requesting more data from source", op_id);
 	state->count = 0;
-	next_browse_id =
-	  grl_media_source_browse (source,
-                                   ui_state->cur_container,
-                                   browse_keys (),
-                                   state->offset, BROWSE_CHUNK_SIZE,
-                                   BROWSE_FLAGS,
-                                   browse_cb,
-                                   state);
-	operation_started (source, next_browse_id, FALSE);
+	switch (state->type) {
+	  case OP_TYPE_BROWSE:
+	    next_op_id =
+	      grl_media_source_browse (source,
+				       ui_state->cur_container,
+				       browse_keys (),
+				       state->offset, BROWSE_CHUNK_SIZE,
+				       BROWSE_FLAGS,
+				       browse_search_query_cb,
+				       state);
+	    break;
+	  case OP_TYPE_SEARCH:
+	    next_op_id =
+	      grl_media_source_search (source,
+				       state->text,
+				       browse_keys (),
+				       state->offset, BROWSE_CHUNK_SIZE,
+				       BROWSE_FLAGS,
+				       browse_search_query_cb,
+				       state);
+	    break;
+	  case OP_TYPE_QUERY:
+	    next_op_id =
+	      grl_media_source_query (source,
+				      state->text,
+				      browse_keys (),
+				      state->offset, BROWSE_CHUNK_SIZE,
+				      BROWSE_FLAGS,
+				      browse_search_query_cb,
+				      state);
+	    break;
+	  case OP_TYPE_MULTI_SEARCH:
+	    /* this shouldn't happen as multiple search has no chunk
+	     * size parameter */
+	    g_warn_if_reached ();
+	    goto operation_finished;
+	    break;
+	}
+	operation_started (source, next_op_id, FALSE);
       } else {
 	/* We browsed all requested elements  */
-	goto browse_finished;
+	goto operation_finished;
       }
     } else {
       /* The operation was cancelled */
-      goto browse_finished;
+      goto operation_finished;
     }
   }
 
   return;
 
- browse_finished:
+ operation_finished:
   g_free (state);
   operation_finished ();
-  GRL_DEBUG ("**** browse finished (%d) ****", browse_id);
+  GRL_DEBUG ("**** operation finished (%d) ****", op_id);
 }
 
 static void
@@ -621,12 +737,13 @@ browse (GrlMediaSource *source, GrlMedia *container)
     clear_panes ();
 
     OperationState *state = g_new0 (OperationState, 1);
+    state->type = OP_TYPE_BROWSE;
     browse_id = grl_media_source_browse (source,
                                          container,
                                          browse_keys (),
                                          0, BROWSE_CHUNK_SIZE,
                                          BROWSE_FLAGS,
-                                         browse_cb,
+                                         browse_search_query_cb,
                                          state);
     operation_started (source, browse_id, FALSE);
   } else {
@@ -976,85 +1093,6 @@ remove_btn_clicked_cb (GtkButton *btn, gpointer user_data)
   }
 }
 
-static void
-search_cb (GrlMediaSource *source,
-	   guint search_id,
-	   GrlMedia *media,
-	   guint remaining,
-	   gpointer user_data,
-	   const GError *error)
-{
-  const gchar *name;
-  GtkTreeIter iter;
-  GdkPixbuf *icon;
-  OperationState *state = (OperationState *) user_data;
-  guint next_search_id;
-  gint type;
-
-  if (error) {
-    g_critical ("Error: %s", error->message);
-  }
-
-  state->count++;
-
-  if (media) {
-    icon = get_icon_for_media (media);
-    name = grl_media_get_title (media);
-    if (GRL_IS_MEDIA_BOX (media)) {
-      gint childcount =
-        grl_media_box_get_childcount (GRL_MEDIA_BOX (media));
-      type = OBJECT_TYPE_CONTAINER;
-      if (childcount != GRL_METADATA_KEY_CHILDCOUNT_UNKNOWN) {
-	name = g_strdup_printf ("%s (%d)", name, childcount);
-      } else {
-	name = g_strconcat (name, " (?)", NULL);
-      }
-    } else {
-      type = OBJECT_TYPE_MEDIA;
-    }
-
-    gtk_list_store_append (GTK_LIST_STORE (view->browser_model), &iter);
-    gtk_list_store_set (GTK_LIST_STORE (view->browser_model),
-			&iter,
-			BROWSER_MODEL_SOURCE, source,
-			BROWSER_MODEL_CONTENT, media,
-			BROWSER_MODEL_TYPE, type,
-			BROWSER_MODEL_NAME, name,
-			BROWSER_MODEL_ICON, icon,
-			-1);
-
-    g_object_unref (media);
-    gdk_pixbuf_unref (icon);
-  }
-
-  if (remaining == 0) {
-    state->offset += state->count;
-    if (!ui_state->multiple &&
-	state->count >= BROWSE_CHUNK_SIZE &&
-	state->offset < BROWSE_MAX_COUNT) {
-      state->count = 0;
-      next_search_id =
-	grl_media_source_search (source,
-                                 state->text,
-                                 browse_keys (),
-                                 state->offset, BROWSE_CHUNK_SIZE,
-                                 BROWSE_FLAGS,
-                                 search_cb,
-                                 state);
-      operation_started (source, next_search_id, FALSE);
-    } else {
-      goto search_finished;
-    }
-  }
-
-  return;
-
- search_finished:
-  g_free (state);
-  operation_finished ();
-  GRL_DEBUG ("**** search finished (%d) ****", search_id);
-}
-
 static void
 search (GrlMediaSource *source, const gchar *text)
 {
@@ -1069,22 +1107,24 @@ search (GrlMediaSource *source, const gchar *text)
   state->text = (gchar *) text;
   if (source) {
     /* Normal search */
+    state->type = OP_TYPE_SEARCH;
     search_id = grl_media_source_search (source,
 					 text,
 					 browse_keys (),
 					 0, BROWSE_CHUNK_SIZE,
 					 BROWSE_FLAGS,
-					 search_cb,
+					 browse_search_query_cb,
 					 state);
   } else {
     /* Multiple search (all sources) */
     multiple = TRUE;
+    state->type = OP_TYPE_MULTI_SEARCH;
     search_id = grl_multiple_search (NULL,
 				     text,
 				     browse_keys (),
 				     BROWSE_MAX_COUNT,
 				     BROWSE_FLAGS,
-				     search_cb,
+				     browse_search_query_cb,
 				     state);
   }
   clear_panes ();
@@ -1104,7 +1144,12 @@ search_btn_clicked_cb (GtkButton *btn, gpointer user_data)
 			SEARCH_MODEL_SOURCE, &source,
 			-1);
     text = gtk_entry_get_text (GTK_ENTRY (view->search_text));
-    search (source, text);
+    /* Special case: empty search means search all */
+    if (text[0] == '\0') {
+      search (source, NULL);
+    } else {
+      search (source, text);
+    }
 
     if (source) {
       g_object_unref (source);
@@ -1123,12 +1168,13 @@ query (GrlMediaSource *source, const gchar *text)
 
   state = g_new0 (OperationState, 1);
   state->text = (gchar *) text;
+  state->type = OP_TYPE_QUERY;
   query_id = grl_media_source_query (source,
                                      text,
                                      browse_keys (),
                                      0, BROWSE_CHUNK_SIZE,
                                      BROWSE_FLAGS,
-                                     search_cb,
+                                     browse_search_query_cb,
                                      state);
   clear_panes ();
   operation_started (source, query_id, FALSE);
@@ -1431,6 +1477,7 @@ ui_setup (void)
 
   GtkActionGroup *actions = gtk_action_group_new ("actions");
   gtk_action_group_add_actions (actions, entries, G_N_ELEMENTS (entries), NULL);
+  gtk_action_group_add_toggle_actions (actions, toggle_entries, G_N_ELEMENTS (toggle_entries), NULL);
 
   GtkUIManager *uiman = gtk_ui_manager_new ();
   gtk_ui_manager_insert_action_group (uiman, actions, 0);
@@ -1597,7 +1644,8 @@ ui_setup (void)
   gchar *col_attributes_md[] = {"text", "text"};
   gint col_model_md[2] = { METADATA_MODEL_NAME, METADATA_MODEL_VALUE};
   col_renders_md[0] = gtk_cell_renderer_text_new ();
-  col_renders_md[1] = gtk_cell_renderer_text_new ();
+  col_renders_md[1] = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
+				    "editable", TRUE, NULL);
   col = gtk_tree_view_column_new ();
   for (i=0; i<2; i++) {
     gtk_tree_view_column_pack_start (col, col_renders_md[i], FALSE);
@@ -1611,6 +1659,17 @@ ui_setup (void)
 
   gtk_container_add (GTK_CONTAINER (scroll_md), view->metadata);
   gtk_container_add (GTK_CONTAINER (view->rpane), scroll_md);
+
+  /* Status bar */
+  view->statusbar = gtk_statusbar_new ();
+  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (view->statusbar), FALSE);
+  view->statusbar_context_id =
+    gtk_statusbar_get_context_id (GTK_STATUSBAR (view->statusbar),
+                                  "changes notification");
+  gtk_container_add_with_properties (GTK_CONTAINER (view->rpane),
+                                     view->statusbar,
+                                     "expand", FALSE, NULL);
+
   gtk_widget_set_size_request (view->metadata,
 			       METADATA_MIN_WIDTH,
 			       METADATA_MIN_HEIGHT);
@@ -1692,6 +1751,70 @@ reset_ui (void)
   show_plugins ();
 }
 
+static gboolean
+remove_notification (gpointer data)
+{
+  gtk_statusbar_remove (GTK_STATUSBAR (view->statusbar),
+                        view->statusbar_context_id,
+                        GPOINTER_TO_UINT (data));
+
+  return FALSE;
+}
+
+static void
+content_changed_cb (GrlMediaSource *source,
+                    GrlMedia *media,
+                    GrlMediaSourceChangeType change_type,
+                    gboolean location_unknown,
+                    gpointer data)
+{
+  const gchar *media_id = grl_media_get_id (media);
+  const gchar *change_type_string = "";
+  const gchar *location_string = "";
+  gchar *message;
+  guint id;
+
+  switch (change_type) {
+  case GRL_CONTENT_CHANGED:
+    change_type_string = "changed";
+    break;
+  case GRL_CONTENT_ADDED:
+    change_type_string = "been added";
+    break;
+  case GRL_CONTENT_REMOVED:
+    change_type_string = "been removed";
+    break;
+  }
+
+  if (location_unknown) {
+    location_string = "(unknown place)";
+  }
+
+  if (GRL_IS_MEDIA_BOX (media)) {
+    message =
+      g_strdup_printf ("%s: container '%s' has %s%s",
+                       grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)),
+                       media_id? media_id: "root",
+                       change_type_string,
+                       location_string);
+  } else {
+    message =
+      g_strdup_printf ("%s: element '%s' has %s",
+                       grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)),
+                       media_id,
+                       change_type_string);
+  }
+
+  id = gtk_statusbar_push (GTK_STATUSBAR (view->statusbar),
+                           view->statusbar_context_id,
+                           message);
+
+  g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
+                         remove_notification,
+                         GUINT_TO_POINTER (id));
+  g_free (message);
+}
+
 static void
 source_added_cb (GrlPluginRegistry *registry,
 		 GrlMediaPlugin *source,
@@ -1715,6 +1838,16 @@ source_added_cb (GrlPluginRegistry *registry,
   /* Also refresh the search combos */
   search_combo_setup ();
   query_combo_setup ();
+
+  /* Check for changes in source (if supported) */
+  if (ui_state->changes_notification &&
+      (grl_metadata_source_supported_operations (GRL_METADATA_SOURCE (source)) &
+       GRL_OP_NOTIFY_CHANGE)) {
+    if (grl_media_source_notify_change_start (GRL_MEDIA_SOURCE (source), NULL)) {
+      g_signal_connect (GRL_MEDIA_SOURCE (source), "content-changed",
+                        G_CALLBACK (content_changed_cb), NULL);
+    }
+  }
 }
 
 static void
